﻿using System.Text;
using System;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using CaliBetManagementSystem;

internal class Program
{
    private static void Main(string[] args)
    {
        HtmlHandler htmlHandler = new HtmlHandler();
        while (true)
        {
            Console.WriteLine("請輸入帳號:");
            string account = Console.ReadLine();
            Console.WriteLine("請輸入密碼:");
            string password = Console.ReadLine();
            if (htmlHandler.Login(account, password))
            {
                //依序輸入
                //1.檢索開始時間
                //2.檢索結束時間
                //3.頁數
                //4.每頁資料數量
                htmlHandler.GetWinLoss(new DateTime(2023, 7, 1, 12, 0, 0), new DateTime(2023, 8, 1, 12, 0, 0), 1, 100);
                Console.ReadLine();
            }
        }
    }

    public class HtmlHandler
    {
        private string? _accessToken;
        private HttpClient _httpClient;
        private string _username;
        private string _password;
        private string _agentID;

        public HtmlHandler()
        {
            var handler = new HttpClientHandler();
            handler.ClientCertificateOptions = ClientCertificateOption.Manual;
            handler.ServerCertificateCustomValidationCallback =
                (httpRequestMessage, cert, cetChain, policyErrors) =>
                {
                    return true;
                };
            _httpClient = new HttpClient();
        }

        /// <summary>
        /// 登入方法
        /// </summary>
        /// <param name="username">帳號</param>
        /// <param name="password">密碼</param>
        public bool Login(string username, string password)
        {
            JObject jObject = new JObject();
            jObject["username"] = _username = username;
            _password = password;
            jObject["password"] = password.ToMD5(username);
            jObject["refer"] = "https://ams.cali9999.net/";
            var res = _httpClient.PostAsync($"https://ams.cali9999.net/service/auth?t={DateTime.Now.Millisecond}", new StringContent(JsonConvert.SerializeObject(jObject), Encoding.UTF8, "application/json")).Result;
            if (res.IsSuccessStatusCode)
            {
                var content = JsonConvert.DeserializeObject<JObject>(res.Content.ReadAsStringAsync().Result);
                if (content is not null)
                {
                    Console.WriteLine(content["messages"].ToString());
                    //登入成功
                    if (content["resultCode"] is not null && (int)content["resultCode"] == 0)
                    {
                        //設定accesstoken
                        _accessToken = content["data"]["token"].ToString();
                        _httpClient.DefaultRequestHeaders.Remove("X-Token");
                        _httpClient.DefaultRequestHeaders.Add("X-Token", $"{_accessToken}");

                        //獲取agentID
                        res = _httpClient.GetAsync($"https://ams.cali9999.net/service/operator?t={DateTime.Now.Millisecond}").Result;

                        content = JsonConvert.DeserializeObject<JObject>(res.Content.ReadAsStringAsync().Result);

                        if (content is not null)
                        {
                            _agentID = content["data"]["agentId"].ToString();
                        }

                        return true;
                    }
                    return false;
                }
            }
            return false;
        }

        /// <summary>
        /// 代理輸贏報表方法
        /// </summary>
        /// <param name="startTime">檢索的開始時間</param>
        /// <param name="endTime">檢索的結束時間</param>
        /// <param name="pageNum">頁數，與下面數值相關聯</param>
        /// <param name="numPerPage">每頁多少資料，與上面數值相關聯</param>
        public void GetWinLoss(DateTime startTime, DateTime endTime, int pageNum, int numPerPage)
        {
            //獲取總輸贏報表資訊
            var res = _httpClient.GetAsync($"https://ams.cali9999.com/service/reports/profit/{_agentID}/summary?t={DateTime.Now.Millisecond}&agentId={_agentID}&startTime={((DateTimeOffset)startTime).ToUnixTimeMilliseconds()}&endTime={((DateTimeOffset)endTime).ToUnixTimeMilliseconds()}&page.numPerPage={numPerPage}&page.pageNum={pageNum}").Result;
            if (res.IsSuccessStatusCode)
            {
                var totalContent = JsonConvert.DeserializeObject<JObject>(res.Content.ReadAsStringAsync().Result);
                //判斷token有沒有過期
                /*if (totalContent["resultCode"] is not null && (int)totalContent["resultCode"] != 0)
                {
                    Login(_username, _password);
                    GetWinLoss(startTime, endTime, pageNum, numPerPage);
                    return;
                }*/

                var info = totalContent["data"]["total"].ToObject<WinLossSummary>();
                if (info != null)
                {
                    Console.WriteLine($"投注金額:{info.BetAmount} 輸贏:{info.WinOrLossAmount} 有效投注:{info.RebateAmount} 洗碼佣金{info.RebateCommission} 總金額:{info.TotalAmount} 交上線:{info.ActualSettlement} 交上線有效投注:{info.RebateToSuperior}");
                }
            }

            //獲取個別代理資訊
            res = _httpClient.GetAsync($"https://ams.cali9999.net/service/v2/reports/profit/34961/subagents?t={DateTime.Now.Millisecond}&agentId={_agentID}&startTime={((DateTimeOffset)startTime).ToUnixTimeMilliseconds()}&endTime={((DateTimeOffset)endTime).ToUnixTimeMilliseconds()}&page.numPerPage={numPerPage}&page.pageNum={pageNum}").Result;
            if (res.IsSuccessStatusCode)
            {
                var agentContent = JsonConvert.DeserializeObject<JObject>(res.Content.ReadAsStringAsync().Result);
                //判斷token有沒有過期
                /*if (agentContent["resultCode"] is not null && (int)agentContent["resultCode"] != 0)
                {
                    Login(_username, _password);
                    GetWinLoss(startTime, endTime, pageNum, numPerPage);
                    return;
                }*/
                if (agentContent["data"]["agentDetails"]["groups"] != null)
                {
                    foreach (var item in agentContent["data"]["agentDetails"]["groups"])
                    {
                        //名稱詳見類別WinLossMember
                        var info = item["agents"][0]["total"].ToObject<WinLossAgent>();
                        if (info != null)
                        {
                            Console.WriteLine($"名稱:{item["agents"][0]["agent"]["userName"].ToString()} 投注金額:{info.BetAmount} 輸贏:{info.TotalAmount} 有效投注:{info.RebateAmount} 洗碼佣金:{info.RebateCommission} 總金額:{info.TotalAmount} 交上線:{info.ActualSettlement} 交上線有效投注:{info.RebateToSuperior}");
                        }
                    }
                }
            }
        }
    }
}